@@ -21,7 +21,6 @@ import typing as T
 
 
 isAsyncGenerator=lambda x: "async_generator" in str(type(x)) #TODO: howto better ?
-fixBacktip=lambda x: x.replace("`",r"\`")
 
 value=lambda x: x.getValue() if isinstance(x,ReactiveProp) else x
 
@@ -78,7 +77,7 @@ class Tag(metaclass=MyMetaclass):
         return """<%(tag)s%(attrs)s>%(content)s</%(tag)s>""" % dict(
             tag=self.tag.replace("_","-"),
             attrs=" ".join([""]+rattrs) if rattrs else "",
-            content=" ".join([str(i) for i in self.__contents if i is not None and not isinstance(i,bool)]),
+            content=" ".join([str(i) for i in self.__contents if value(i) is not None and not isinstance(i,bool)]),
         )
     def __repr__(self):
         return "<%s>" % self.__class__.__name__
@@ -187,10 +186,6 @@ class ReactiveProp:
     def __str__(self):
         return str(self.getValue())
 
-    def __repr__(self):
-        iid=self._instance.id if hasattr(self._instance,"id") else str(self._instance)
-        return "<ReactiveProp:%s attr=%s of instance=%s>" % (self.__class__.__name__,self._attribut,iid)
-
 
     @property
     def __class__(self):
@@ -209,7 +204,6 @@ class ReactiveProp:
         return value(x) in self.getValue()
 
 
-
     def __getattr__(self,k):
         return getattr(self.getValue(),k)
     def __setattr__(self,k,v):
@@ -225,6 +219,11 @@ class ReactiveProp:
         return self.getValue()(*a,**k)
     #TODO: add a lot of __slot__ ;-)
 
+    def __repr__(self):
+        iid=self._instance.id if hasattr(self._instance,"id") else str(self._instance)
+        return "<ReactiveProp:%s attr=%s of instance=%s>" % (self.__class__.__name__,self._attribut,iid)
+
+
 
 class render:
 
@@ -271,7 +270,7 @@ class Binder:
             # resolve all declaredJsVars from all childs
             pool={}
             for id,o in self.__instance._getChilds().items():
-                pool.update( {o.__class__.__name__:o._declaredJsInputs} )
+                pool.update( o._declaredJsInputs )
             #/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\
 
             def _(*args,**kargs):
@@ -279,7 +278,7 @@ class Binder:
                     args=[value(i) for i in args]
                     kargs={k:value(v) for k,v in kargs.items()}
 
-                return "self.bindUpdate('%s',GID,'%s',%s,%s,%s)" % (self.__instance.id,name,jjs(args),jjs(kargs),jjs(pool))
+                return "callEvent('%s','%s',%s,%s,%s)" % (self.__instance.id,name,jjs(args),jjs(kargs),jjs(pool))
             return _
         else:
             raise Exception("Unknown method '%s' in '%s'"%(name,self.__instance.__class__.__name__))
@@ -330,8 +329,18 @@ class GTag:
         log("INIT",repr(self))
 
         #/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\
+        if self.parent is None:
+            self.main._localInputs={}
+
         signature = inspect.signature( self.init )
         self._declaredJsInputs={k: v.default for k, v in signature.parameters.items() if type(v.default)==bytes}
+        for k,v in self._declaredJsInputs.items():
+            if k not in self.main._localInputs:
+                self.main._localInputs[k]=None         # init js props at null
+            rp=ReactiveProp( self.main._localInputs,k)
+            #~ self._data[k]=rp
+            super().__setattr__(k,rp)
+
         #/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\
 
         self.init(*self._args,**self._kargs)
@@ -417,6 +426,7 @@ class GTag:
         gtag = self.__class__(*self._args,**self._kargs,parent=None) # parent=None, will avoid guess parent ! (it makes sense, because you can clone only mains)
         gtag.__dict__.update(props)
         gtag._scripts=[]
+        gtag._localInputs = self._localInputs
         gtag.init(*self._args,**self._kargs)
         gtag._rebuild()
         log("^^^ CLONED ^^^",repr(self),"-->",repr(gtag))
@@ -461,10 +471,10 @@ class GTag:
                 await self._call
             elif isAsyncGenerator(self._call):
                 async for _ in self._call:
-                    assert _ is None, "wtf?"
+                    assert _ is None, "wtf (event returns something)?"
                     yield
             else:
-                raise Exception("Not implemented (calling a sync/start function)")
+                raise Exception("Not implemented (calling a (sync)start function)")
         #=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
 
     def exit(self,v=None): pass # overriden by run/runcef/serve
@@ -555,19 +565,26 @@ class GTag:
         return ";".join(ll)
 
 
-    def _update(self) -> dict:
+    def _render(self) -> dict:
+
+        #/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\
+        # resolve all declaredJsVars from all childs
+        pool={}
+        for id,o in self._getChilds().items():
+            pool.update( o._declaredJsInputs )
+        #/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\
+
         h=str(self)
-        s=self._getScripts()
+
         log(">>>UPDATE:",repr(self))
         log(self._tree())
 
-        return dict( script="""
-(function (id,content) {
-    let o=document.querySelector(id);
-    if(o) o.outerHTML=content;
-    else document.body.innerHTML=content;
-})("#%s",`%s`); %s
-        """ % (self.id, fixBacktip(h),s))
+        return dict(
+            id       = self.id,
+            content  = h,
+            scripts  = self._getScripts(),
+            exchange = jjs(pool),
+        )
 
     def run(self,*a,start=None,**k) -> any:
         """ Run as Guy App (using Chrome) """
@@ -619,11 +636,24 @@ class GTagApp(guy.Guy):
             if(!sessionStorage["gtag"]) sessionStorage["gtag"]=Math.random().toString(36).substring(2);
             var GID=sessionStorage["gtag"];
             async function getSessionId() {return GID}
+
+            async function execute(id,content,scripts,exchange) {
+                document.querySelector("#"+id).outerHTML=content;
+                eval(scripts)
+                return eval("x="+exchange);
+            }
+
+            async function callEvent(id,event,a,k,e) {
+                let r=await self.bindUpdate(id,GID,event,a,k,e);
+                await execute( r.id, r.content, r.scripts, r.exchange );
+            }
         </script>
         %s
     </head>
     <body>
+        <div id="gtag">
             <script src="guy.js"></script>
+        </div>
     </body>
 </html>""" % "\n".join([str(h) for h in hh])
 
@@ -643,16 +673,25 @@ class GTagApp(guy.Guy):
         log(">>>SERVE",repr(gtag))
         log(gtag._tree())
 
-        await self.js.eval( gtag.bind._start()+";"+gtag._getScripts() )
+        r=gtag._render()
+        scripts=r["scripts"]+";"+gtag.bind._start()
+        #/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\
+        gtag.main._localInputs.update( await self.js.execute( "gtag", r["content"], scripts, r["exchange"] ) )
+        #/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\
+
+
 
 
     async def bindUpdate(self,id:str,gid:str,method:str,args,kargs,jsArgs={}):
         """ inner (js exposed) guy method, called by gtag.bind.<method>(*args) """
-
-        async def forceUpdate(g):
+        async def asyncRender(g):
             g._rebuild(clearScripts=False)
             log(">>>Force UPDATE:",repr(g))
-            await self.js.eval( g._update()["script"] )
+            r=g._render()
+            #/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\
+            gtag.main._localInputs.update(await self.js.execute( r["id"], r["content"], r["scripts"], r["exchange"] ))
+            #/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\
+            g._clearScripts()
 
         if self._ses is None:
             gtag=self._originalGTag
@@ -660,21 +699,16 @@ class GTagApp(guy.Guy):
             gtag=self._ses[gid]
 
         #////////////////////////////////////////////////////////////////// THE MAGIC TODO: move to gtag
-        obj=gtag._getRef(id)
-
-        log("BINDUPDATE on",repr(gtag),"----->",repr(obj),"%s(%s %s)"% (method,args,kargs))
-        proc=getattr(obj,method)
 
         #/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\
         # dispatch jsArgs in gtag childs
         #/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\
-        for o in gtag._getChilds().values():
-            defs=jsArgs.get( o.__class__.__name__,{})
-            if defs:
-                for jsk in o._declaredJsInputs.keys():
-                    if jsk in defs:
-                        setattr(o,jsk,defs[jsk])
+        gtag.main._localInputs.update(jsArgs)
         #/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\
+        obj=gtag._getRef(id)
+
+        log("BINDUPDATE on",repr(gtag),"----->",repr(obj),"%s(%s %s)"% (method,args,kargs))
+        proc=getattr(obj,method)
 
         if Capacity(proc).has(render.local):
             toRender=obj
@@ -697,12 +731,17 @@ class GTagApp(guy.Guy):
                 async for _ in rep: # could use yielded thing to update all or local ?!
                     assert _ is None, "wtf (event returns something)?"
                     if toRender:
-                        await forceUpdate(toRender)
+                        await asyncRender(toRender)
+            elif isinstance(rep, types.GeneratorType):
+                for _ in rep:
+                    assert _ is None, "wtf (event returns something)?"
+                    if toRender:
+                        await asyncRender(toRender)
             else:
                 raise Exception("wtf (event returns something)?")
 
         if toRender:
             toRender._rebuild(clearScripts=False)
-            return toRender._update()
+            return toRender._render()
         #////////////////////////////////////////////////////////////////// THE MAGIC
 
